package jaredbgreat.dldungeons.planner;


/*This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/	


import jaredbgreat.dldungeons.builder.Builder;
import jaredbgreat.dldungeons.pieces.Spawner;
import jaredbgreat.dldungeons.pieces.chests.BasicChest;
import jaredbgreat.dldungeons.pieces.entrances.AbstractEntrance;
import jaredbgreat.dldungeons.pieces.entrances.SimpleEntrance;
import jaredbgreat.dldungeons.pieces.entrances.SpiralStair;
import jaredbgreat.dldungeons.pieces.entrances.TopRoom;
import jaredbgreat.dldungeons.planner.mapping.MapMatrix;
import jaredbgreat.dldungeons.rooms.Room;
import jaredbgreat.dldungeons.rooms.RoomList;
import jaredbgreat.dldungeons.themes.BiomeLists;
import jaredbgreat.dldungeons.themes.Degrees;
import jaredbgreat.dldungeons.themes.Sizes;
import jaredbgreat.dldungeons.themes.Theme;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Random;

import net.minecraft.world.World;
import net.minecraft.world.biome.BiomeGenBase;


public class Dungeon {
	
	public Theme theme;
	public Random random;
	public BiomeGenBase biome;
	
	public MapMatrix map;
	public Node[] nodes;
	public int numNodes;
	public int roomCount;
	public int entrancePref;
	
	public int baseHeight;
	
	public RoomList rooms;
	public ArrayList<Room> planter;
	public ArrayList<Room> grower;
	//TODO: Other Components
	
	// Planning parameters
	public Sizes   size;
	// Not sure if I'll use all of them...	
	public Degrees outside;		// Roofless rooms (also wall-less, but may have fences)
	public Degrees liquids;		// Quantity of water / lava pools
	public Degrees subrooms;	// Rooms budding of from the main one
	public Degrees islands;		// Rooms inside rooms	
	public Degrees pillars;		// Uh, pillars / columns, duh!
	public Degrees fences;		// Uh, fences, duh!
	public Degrees symmetry;	// How symmetrical rooms are (technically, chance of axis mirroring)
	public Degrees variability;	// Inconsistency, that chance of using a different style in some place
	public Degrees degeneracy;	// Chance of walls / ceilings not spawning over airblocks (idea taken from Greymerk)
	public Degrees complexity;	// Basically how many shape primitives to add; depth of added place seeds
	public Degrees verticle;	// How many height change and how much height change
	public Degrees entrances;	// Ways in and outS
	public Degrees bigRooms;
	
	// Default blocks
	public int wallBlock1;
	public int floorBlock;
	public int cielingBlock;
	public int stairBlock;
	public int stairSlab;
	public int fenceBlock;
	public int cornerBlock;
	public int liquidBlock;
	
	
	@Override
	public void finalize() throws Throwable {
		for(int i = 0; i < nodes.length; i++) nodes[i] = null;
		for(Room room: rooms) room.finalize();
		rooms.clear();
		rooms = null;
		planter = grower = null;
		nodes = null;
		theme =  null;
		random = null;
		biome = null;
		map = null;		
		size = null;	
		outside = null;
		liquids = null;
		subrooms = null;
		islands = null;	
		pillars = null;
		symmetry = null;
		variability = null;
		degeneracy = null;
		complexity = null;
		verticle = null;
		entrances = null;
		bigRooms = null;
		super.finalize();
	}

	
	public Dungeon(Random random, BiomeGenBase biome, World world, int chunkX, int chunkZ) throws Throwable {		
		//this.random = random;
		this.random = new Random(random.nextLong());
		this.biome = biome;
		theme = BiomeLists.getTheme(biome, random);
		applyTheme();
		entrancePref = random.nextInt(3);
		
		
		wallBlock1 = theme.walls[random.nextInt(theme.walls.length)];
		floorBlock = theme.floors[random.nextInt(theme.floors.length)];
		cielingBlock = theme.ceilings[random.nextInt(theme.ceilings.length)];
		fenceBlock = theme.fencing[random.nextInt(theme.fencing.length)];
		cornerBlock = theme.pillarBlock[random.nextInt(theme.pillarBlock.length)];
		liquidBlock = theme.liquid[random.nextInt(theme.liquid.length)];
		
		rooms = new RoomList(size.maxRooms + 1);
		planter = new ArrayList<Room>();
		map = new MapMatrix(size.width, world, chunkX, chunkZ);
		numNodes = random.nextInt(size.maxNodes - size.minNodes + 1) + size.minNodes + 1;
		nodes = new Node[numNodes];
		makeNodes();
		connectNodes();
		growthCycle();
	}
	
	
	private void applyTheme() {
		size  	 	= theme.sizes.select(random);
		outside 	 	= theme.outside.select(random);
		liquids	 	= theme.liquids.select(random);
		subrooms 	= theme.subrooms.select(random);
		islands	 	= theme.islands.select(random);
		pillars	 	= theme.pillars.select(random);
		symmetry 	= theme.symmetry.select(random);
		variability = theme.variability.select(random);
		degeneracy	= theme.degeneracy.select(random);
		complexity	= theme.complexity.select(random);
		verticle	= theme.verticle.select(random);
		entrances	= theme.entrances.select(random);
		fences		= theme.fences.select(random);
		baseHeight  = random.nextInt(theme.maxY - theme.minY) + theme.minY;
	}
	
	
	void makeNodes() {
		int height = baseHeight;
		for(int i = 0; i < numNodes; i++) {
			nodes[i] = new Node(random.nextInt(size.width), height, random.nextInt(size.width), random, this);
		}
		//System.out.println("[DLDUNGEONS] " + numNodes + " nodes");		
	}
	
	
	void connectNodes() throws Throwable {
		Node first, other;
		for(int i = 0; i < nodes.length; i++) {
			first = nodes[i];
			if(first == null) continue;
			if(first.hubRoom == null) continue;
			for(int j = i + 1; j < nodes.length; j++) {
				other = nodes[j];
				if(other == null) continue;
				if(other.hubRoom == null) continue;
				if(rooms.size() >= size.maxRooms) return;
				if(other != first) {
					new Route(first, other).drawConnections(this);
				}
			}
		}
	}
	
	
	void makeMoreRooms() {
		while(rooms.size() <= size.maxRooms) {
				Room made;
				int height = baseHeight;
				int x = random.nextInt(size.width);
				int z = random.nextInt(size.width);
				int xdim = random.nextInt(size.maxRoomSize - 5) + 6;
				int zdim = random.nextInt(size.maxRoomSize - 5) + 6;
				if(bigRooms.use(random)) {
					xdim += random.nextInt((size.maxRoomSize / 2)) + (size.maxRoomSize / 2);
					zdim += random.nextInt((size.maxRoomSize / 2)) + (size.maxRoomSize / 2);
				}
				int ymod = (xdim <= zdim) ? (int) Math.sqrt(xdim) : (int) Math.sqrt(zdim);
				int roomHeight = random.nextInt((verticle.value / 2) + ymod + 1) + 2;
				made = new PlaceSeed(x, height, z).growRoom(xdim, zdim, roomHeight, this, null, null);
			}
		}
	
	
	public void growthCycle() {
		//System.out.println("Running growthCycle.");
		int depth = subrooms.value + 1;
		int limit = (size.maxRooms * (depth + 1)) / 5;
		boolean doMore = true;
		do {
			depth--;
			doMore = false;
			grower = planter;
			Collections.shuffle(grower, random);
			planter = new ArrayList<Room>();
			for(Room room : grower) {
				limit--; 
				if(rooms.size() >= size.maxRooms) return;
				if(room.plantChildren(this)) {
					//System.out.println("Added side room.");
					doMore = true;
				}
			} 
		} while(doMore && (depth > 0) && (limit > 0));	
	}
	
	
	public void addSpawners() {
		// TODO: Separate methods for chests and entrances
		int shiftX = (map.chunkX * 16) - (map.room.length / 2) + 8;
		int shiftZ = (map.chunkZ * 16) - (map.room.length / 2) + 8;
		for(Room room : rooms) {
			for(Spawner  spawner : room.spawners) {
				Builder.placeSpawner(map.world, shiftX + spawner.x, spawner.y, shiftZ + spawner.z, spawner.mob);
			}
			for(BasicChest  chest : room.chests) {
				chest.place(map.world, shiftX + chest.mx, chest.my, shiftZ + chest.mz, random);
			}
			// TODO: Method for better selecting an entrance
			if(room.hasEntrance) addEntrances(room);
			}
		}
	
	
	public void addEntrances(Room room) {
		int entrance;
		if(variability.use(random)) entrance = random.nextInt(3);
		else entrance = entrancePref; 
		switch (entrance) {
		case 0:
			new SpiralStair((int)room.realX, room.floorY, (int)room.realZ).build(this, map.world);
			break;
		case 1:
			new TopRoom((int)room.realX, room.floorY, (int)room.realZ).build(this, map.world);
			break;
		case 2:
		default:
			new SimpleEntrance((int)room.realX, room.floorY, (int)room.realZ).build(this, map.world);
			break;
		}		
	}
	
}


