package jaredbgreat.dldungeons;


/*This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/	


import jaredbgreat.dldungeons.builder.Builder;
import jaredbgreat.dldungeons.themes.Theme;
import jaredbgreat.dldungeons.themes.ThemeReader;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collections;

import net.minecraft.block.Block;
import net.minecraft.entity.EntityList;
import net.minecraft.entity.EntityLiving;
import net.minecraft.item.Item;
import net.minecraftforge.common.config.Configuration;

public class ConfigHandler {
	
	private static File mainConfig;
	private static File themesDir;
	private static File listsDir;
	
	private static final int DEFAULT_SCALE  = 6;
	private static final int DEFAULT_MINXZ  = 16;
	private static final int DEFAULT_DIF    = 3;
	
	private static final int[] DEFAULT_DIMS = {0, -1};

	private static boolean writeLists;	
	
	private static final boolean DEFAULT_NODEND = true;
	private static final boolean DEFAULT_WRITE_LISTS = false;
	private static final boolean DEFAULT_NATURAL_SPAWN = true;
	private static final boolean DEFAULT_OBEY_RULE = true;
	
	protected static boolean naturalSpawn = DEFAULT_NATURAL_SPAWN;	
	protected static boolean obeyRule = DEFAULT_OBEY_RULE;
	
	public static Difficulty difficulty;

	
	public static void init(File file) {
		mainConfig = file;
		Configuration config = new Configuration(file);
		config.load();
		
		int freqScale = config.get("General", "FrequencyScale", DEFAULT_SCALE).getInt();
		if((freqScale > 30) || (freqScale < 1)) freqScale = DEFAULT_SCALE;
		GenerationHandler.setFrequency(freqScale);
		System.out.println("[DLDUNGEONS] Frequency Scaling Factor Set To: " + freqScale);
		
		int minXZ = config.get("General", "MinChunkXY", DEFAULT_MINXZ).getInt();
		if(minXZ < 0) freqScale = DEFAULT_MINXZ;
		GenerationHandler.setMinXZ(minXZ);
		System.out.println("[DLDUNGEONS] Minimum X Factor Set To: " + minXZ);
		
		int diff = config.get("General", "Difficulty", DEFAULT_DIF).getInt();
		if((diff < 0) || (diff > 5)) freqScale = DEFAULT_DIF;
		parseDiff(diff);
		System.out.println("[DLDUNGEONS] Difficulty set to: " + difficulty.label);
		
		int[] dims = config.get("General", "Allowed Dimensions", DEFAULT_DIMS).getIntList();
		GenerationHandler.setDimensions(dims);
		System.out.print("[DLDUNGEONS] Dungeons will spawn in these dimensions: ");
		for(int i = 0; i < dims.length; i++) System.out.print(dims[i] + ", ");
		System.out.println();
		
		boolean noEnd = config.get("General", "NotInEnd", DEFAULT_NODEND).getBoolean(true);
		GenerationHandler.setEnd(noEnd);
		System.out.println("[DLDUNGEONS] NoEnd set to: " + noEnd);
		
		naturalSpawn = config.get("General", "SpawnWithWordgen", DEFAULT_NATURAL_SPAWN).getBoolean(true);
		System.out.println("[DLDUNGEONS] Will spawn dungeons in with world generation? " + naturalSpawn);
		
		obeyRule = config.get("General", "ObeyFeatureSpawningRule", DEFAULT_OBEY_RULE).getBoolean(true);
		System.out.println("[DLDUNGEONS] Will spawn dungeons even with structures disabled? " + !obeyRule);
		
		writeLists = config.get("General", "ExportLists", DEFAULT_WRITE_LISTS).getBoolean(false);
		System.out.println("[DLDUNGEONS] Will export item, block, and mob lists? " + writeLists);
		
		Theme.addDefaultTheme();
		openThemesDir();
		config.save();
	}
	
	
	public static void generateLists() {
		if(!writeLists) return;
		listsDir = new File(themesDir.toString() + File.separator + "lists");
		
		if(!listsDir.exists()) {
			listsDir.mkdir();
		} 		
		if(!listsDir.exists()) {
			System.out.println("[DLDUNGEONS] Warning: Could not create " 
						+ listsDir + ".");
		} else if (!listsDir.isDirectory()) {
			System.out.println("[DLDUNGEONS] Warning: " + listsDir 
					+ " is not a directory (folder); no themes loaded.");
		} else {		
			listEntities();
			listItems();
			listBlocks();
		}
	}
	
	
	public static void listEntities() {	
		ArrayList<String> mobNames = new ArrayList<String>();
		mobNames.addAll(EntityList.stringToClassMapping.keySet());
		Collections.sort(mobNames);
		BufferedWriter outstream = null;
		File moblist = new File(listsDir.toString() + File.separator + "mobs.txt");
		if(moblist.exists()) moblist.delete(); 
		try {
			outstream = new BufferedWriter(new 
					FileWriter(moblist.toString()));
			
			if(outstream == null) throw new IOException();

			for(String name : mobNames){ 
				Class A = (Class)EntityList.stringToClassMapping.get(name);
				//System.out.println(EntityList.stringToClassMapping.get(name));
				//System.out.println(A);
				if(EntityLiving.class.isAssignableFrom(A) && !Modifier.isAbstract(A.getModifiers())) {
					//System.out.println("[DLDUNGEONS] Found living entity " + (String)name);
					outstream.write((String)name);
					outstream.newLine();
				}
			}
			
			if(outstream != null) outstream.close();
		} catch (IOException e) {

			System.err.println("[DLDUNGEONS] Error: Could not write file mobs.txt");
			e.printStackTrace();
		} 
	}
	
	
	public static void listItems() {	
		BufferedWriter outstream = null;
		File itemlist = new File(listsDir.toString() + File.separator + "itmes.txt");
		if(itemlist.exists()) itemlist.delete(); 
		try {
			outstream = new BufferedWriter(new 
					FileWriter(itemlist.toString()));
			
			if(outstream == null) throw new IOException();
			
			for(Object item : Item.itemRegistry){ 
				String name = Item.itemRegistry.getNameForObject(item);
				if(true) {
					//System.out.println("[DLDUNGEONS] Found item " + name);
					outstream.write(name);
					outstream.newLine();
				}
			}
			
			if(outstream != null) outstream.close();
		} catch (IOException e) {
			System.err.println("[DLDUNGEONS] Error: Could not write file items.txt");
			e.printStackTrace();
		}		
	}
	
	
	public static void listBlocks() {	
		BufferedWriter outstream = null;
		File itemlist = new File(listsDir.toString() + File.separator + "blocks.txt");
		if(itemlist.exists()) itemlist.delete(); 
		try {
			outstream = new BufferedWriter(new 
					FileWriter(itemlist.toString()));
			
			if(outstream == null) throw new IOException();	
			
			for(Object block : Block.blockRegistry){ 
				String name = Block.blockRegistry.getNameForObject(block);
				if(true) {
					//System.out.println("[DLDUNGEONS] Found item " + name);
					outstream.write(name);
					outstream.newLine();
				}
			}
			
			if(outstream != null) outstream.close();
		} catch (IOException e) {
			System.err.println("[DLDUNGEONS] Error: Could not write file blocks.txt");
			e.printStackTrace();
		}		
	}
	
	
	private static void openThemesDir() {
		// This will open the directory for theme data; it will not load themes
		// as that must be done post-init incase blocks / mobs / items from
		// other mobs are requested by user themes.
		String themesDirName = mainConfig.getParentFile() + File.separator 
				+ "DLDungeonThemes" + File.separator;
		themesDir = new File(themesDirName);
		if(!themesDir.exists()) {
			themesDir.mkdir();
		} 
		
		if(!themesDir.exists()) {
			System.out.println("[DLDUNGEONS] Warning: Could not create " + themesDirName + ".");
		} else if (!themesDir.isDirectory()) {
			System.out.println("[DLDUNGEONS] Warning: " + themesDirName 
					+ " is not a directory (folder); no themes loaded.");
		} else ThemeReader.setThemesDir(themesDir);		
	}
	
	
	private static void parseDiff(int diff) {
		switch(diff) {
			case 0:
				difficulty = Difficulty.NONE;
				break;
			case 1:
				difficulty = Difficulty.BABY;
				break;
			case 2:
				difficulty = Difficulty.NOOB;
				break;
			case 4:
				difficulty = Difficulty.HARD;
				break;
			case 5:
				difficulty = Difficulty.NUTS;
				break;
			case 3:
			default:
				difficulty = Difficulty.NORM;
				
				break;
		}
	}

	
}
