package jaredbgreat.dldungeons.themes;

/*This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/	


import java.util.ArrayList;
import java.util.EnumSet;

import net.minecraftforge.common.BiomeDictionary.Type;

public class Theme {
	
	public String name;

	public static Theme classic;
	public static Theme nether;
	public static Theme end;
	
	public EnumSet<Type> biomes = EnumSet.noneOf(Type.class);
	public EnumSet<Type> notIn  = EnumSet.noneOf(Type.class);
	
	public int minY;
	public int maxY;
	public boolean buildFoundation;
	
	//TODO:  Biome restriction (especially important for the Nether)

	// Planning parameters
	public SizeElement sizes;
	// Not sure if I'll use all of them...	
	public Element outside;		// Roofless, wall-less rooms (intend for outdoors in surface dungeons)
	public Element liquids;		// Quantity of water / lava pools
	public Element subrooms;	// Rooms budding of from the main one
	public Element islands;		// Rooms inside rooms	
	public Element pillars;		// Uh, pillars / columns, duh!
	public Element fences;		// Uh, fences, duh!
	public Element symmetry;	// How symmetrical rooms are (technically, chance of axis mirroring)
	public Element variability;	// Inconsistency, that chance of using a different style in some place
	public Element degeneracy;	// Chance of walls / ceilings not spawning over air blocks (idea taken from Greymerk)
	public Element complexity;	// Basically how many shape primitives to add
	public Element verticle;	// How many height change and how much height change
//	public Element roomSets;	// Replacing a large room with a house like division of rooms
	public Element entrances;	// Ways in and out / number of entrance/exit nodes (others are destination / treasure nodes)
		
	//Materials
	public BlockList walls;
	public BlockList floors;
	public BlockList ceilings;
	public BlockList fencing;
	public BlockList liquid;
	public BlockList pillarBlock;	
	
	public ArrayList<String> commonMobs;
	public ArrayList<String> hardMobs;
	public ArrayList<String> bruteMobs;
	public ArrayList<String> eliteMobs;
	public ArrayList<String>[] allMobs;
	
	
	public Theme(int minY, int maxY, boolean buildFoundation, 
				 SizeElement sizes, Element skies, Element liquids, 
				 Element subrooms, Element islands, Element pillars, Element fences, 
				 Element symmetry, Element variability, Element degeneracy, Element complexity, 
				 Element verticle, Element entrances, Element bigRooms,
				 BlockList walls, BlockList floors, BlockList ceilings, BlockList stairs,
				 BlockList stairSlabs, BlockList fencing, BlockList liquid, BlockList corner, 
				 BlockList windowContent, String[] mobs, String[] hardMobs) {
		this.minY = minY;
		this.maxY = maxY;
		this.buildFoundation = buildFoundation;
		this.sizes = sizes;	
		this.outside = skies;
		this.liquids = liquids;
		this.subrooms = subrooms;
		this.islands = islands;	
		this.pillars = pillars;
		this.symmetry = symmetry;
		this.variability = variability;
		this.degeneracy = degeneracy;
		this.complexity = complexity;
		this.verticle = verticle;
		this.entrances = entrances;
//		this.bigRooms = bigRooms;
		this.walls = walls;
		this.floors = floors;
		this.ceilings = ceilings;
		this.fencing = fencing;
		this.liquid = liquid;
		this.pillarBlock = corner;
		this.commonMobs = new ArrayList<String>(mobs.length);
		this.hardMobs = new ArrayList<String>(hardMobs.length);
		for(int i = 0; i < mobs.length; i++) {
			this.commonMobs.add(mobs[i]);
		} for(int i = 0; i < hardMobs.length; i++) {
			this.hardMobs.add(hardMobs[i]);
		}
	}
	
	
	public Theme() {
		minY = 10;
		maxY = 50;
		buildFoundation = true;
		sizes = new SizeElement(2, 5, 10, 5, 1);	
		outside = new Element(25,  0,  0,  0,  0,  0);
		liquids = new Element(1,  30, 50, 20, 10,  0);
		subrooms = new Element(20,  0, 50, 45,  5,  0);
		islands = new Element(5,  50, 10, 50, 20,  0);	
		pillars = new Element(5,  30, 60, 40, 20,  0);
		symmetry = new Element(5,  15, 30, 75, 25,  0);
		variability = new Element(5,  10, 25, 75, 50, 25);
		degeneracy = new Element(50,  5, 15, 50, 10,  0);
		complexity = new Element(5,  10, 25, 75, 15,  0);
		verticle = new Element(5,  10, 25, 20, 10,  0);
		entrances = new Element(2,   5, 25, 50, 15,  3);
		fences = new Element(15, 25, 55, 15,  0, 0);
//		bigRooms = new Element(50,  45, 5, 0, 0,  0);
		walls = new BlockList(new String[]{"stonebrick", "cobblestone", "sandstone", "brick_block"});
		floors = new BlockList(new String[]{"stonebrick", "cobblestone", "dirt", "stone"});
		ceilings = new BlockList(new String[]{"stonebrick", "cobblestone", "planks", "double_stone_slab"});
		fencing = new BlockList(new String[]{"cobblestone_wall", "fence"});
		liquid = new BlockList(new String[]{"water", "lava"});
		pillarBlock = new BlockList(new String[]{"stonebrick", "cobblestone", "sandstone", "planks", "double_stone_slab"});
		String[] mobs = new String[]{"Zombie", "Skeleton", "Spider"};
		String[] hard = new String[]{"Creeper", "Enderman", "Witch", "CaveSpider"};
		commonMobs = new ArrayList<String>(mobs.length);
		hardMobs = new ArrayList<String>(hard.length);
		for(int i = 0; i < mobs.length; i++) {
			this.commonMobs.add(mobs[i]);
		} for(int i = 0; i < hard.length; i++) {
			this.hardMobs.add(hard[i]);
		}
	}
	
	
	/*--------------------------------------*
	 * 			ADD THEMES                   *
	 ---------------------------------------*/
	
	
	public static void addDefaultTheme() {		
		String[] mobs;
		String[] hard;
		
		// Default theme
		classic = new Theme();
		classic.fixMobs();
		
		// Nether theme
		nether = new Theme();
		nether.minY = 32;
		nether.maxY = 64;
		nether.walls = new BlockList(new String[]{"nether_brick"});
		nether.floors = new BlockList(new String[]{"nether_brick"});
		nether.ceilings = new BlockList(new String[]{"nether_brick"});
		nether.liquid = new BlockList(new String[]{"lava"});
		nether.pillarBlock = new BlockList(new String[]{"nether_brick", "obsidian", "nether_brick", "obsidian", "quartz_block"});
		nether.fencing = new BlockList(new String[]{"nether_brick_fence"});
		nether.liquids = new Element(0, 0, 5, 85, 10, 0);
		nether.verticle = new Element(0, 0, 5, 55, 15, 0);
		nether.outside = new Element(5, 10, 15, 5, 0, 0);
		nether.pillars = new Element(5, 15, 10, 5, 0, 0);
		nether.degeneracy = new Element(10, 0, 0, 0, 0, 0);
		nether.entrances = new Element(10, 0, 0, 0, 0, 0);
		mobs = new String[]{"LavaSlime", "PigZombie", "Blaze"};
		hard = new String[]{"Blaze", "Skeleton"};
		nether.commonMobs = new ArrayList<String>(mobs.length);
		nether.hardMobs = new ArrayList<String>(hard.length);
		for(int i = 0; i < mobs.length; i++) {
			nether.commonMobs.add(mobs[i]);
		} for(int i = 0; i < hard.length; i++) {
			nether.hardMobs.add(hard[i]);
		}
		nether.bruteMobs = nether.hardMobs;
		nether.eliteMobs = new ArrayList<String>();
		nether.fixMobs();
		
		end = new Theme();
		end.minY = 30;
		end.maxY = 60;
		end.walls = new BlockList(new String[]{"endstone"});
		end.floors = new BlockList(new String[]{"endstone"});
		end.ceilings = new BlockList(new String[]{"endstone"});
		end.liquid = new BlockList(new String[]{"air"});
		end.pillarBlock = new BlockList(new String[]{"endstone", 
				"obsidian", "obsidian", "obsidian", "glowstone"});
		end.liquids = new Element(5, 0, 0, 0, 0, 0);
		end.verticle = new Element(0, 0, 5, 55, 15, 0);
		end.outside = new Element(5, 10, 15, 5, 0, 0);
		end.pillars = new Element(0, 5, 15, 10, 5, 0);
		end.degeneracy = new Element(10, 0, 0, 0, 0, 0);
		end.entrances = new Element(10, 0, 0, 0, 0, 0);
		end.commonMobs = new ArrayList<String>(mobs.length);
		end.hardMobs = new ArrayList<String>(hard.length);
		end.bruteMobs = new ArrayList<String>(hard.length);
		end.eliteMobs = new ArrayList<String>(hard.length);
		end.commonMobs.add("Enderman");
		end.hardMobs.add("Enderman");
		end.bruteMobs.add("Enderman");
		end.eliteMobs.add("WitherBoss");
		end.fixMobs();
	}
	
	
	public void fixMobs() {
		if(hardMobs == null) hardMobs = commonMobs;
		else if(hardMobs.isEmpty()) hardMobs = commonMobs;
		if(bruteMobs == null) bruteMobs = hardMobs;
		else if(bruteMobs.isEmpty()) bruteMobs = hardMobs;
		if(eliteMobs == null) eliteMobs = bruteMobs;
		else if(eliteMobs.isEmpty()) eliteMobs = bruteMobs;
		allMobs = new ArrayList[4];
		allMobs[0] = commonMobs;
		allMobs[1] = hardMobs;
		allMobs[2] = bruteMobs;
		allMobs[3] = eliteMobs;
	}
	
	
	public void biomeRegister() {
		BiomeLists.registerTheme(this);
		BiomeLists.removeTheme(this);
	}
	
	
	@Override
	public String toString() {
		if(name == null) return "null";
		return name;
	}
	
}
